# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: c3loc/proto/secbeacon.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='c3loc/proto/secbeacon.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1b\x63\x33loc/proto/secbeacon.proto\"K\n\x10\x45ncryptedPayload\x12\n\n\x02id\x18\x01 \x01(\x0c\x12\r\n\x05nonce\x18\x02 \x01(\x0c\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\x12\x0b\n\x03tag\x18\x04 \x01(\x0c\"\x12\n\x10PlaintextPayload\"\x9f\x01\n\x0fSecBeaconReport\x12\r\n\x05\x63lock\x18\x01 \x01(\x04\x12&\n\tencrypted\x18\x02 \x01(\x0b\x32\x11.EncryptedPayloadH\x00\x12&\n\tplaintext\x18\x03 \x01(\x0b\x32\x11.PlaintextPayloadH\x00\x12\x13\n\x0b\x64istance_cm\x18\x04 \x01(\r\x12\x10\n\x08variance\x18\x05 \x01(\rB\x06\n\x04\x64\x61tab\x06proto3'
)




_ENCRYPTEDPAYLOAD = _descriptor.Descriptor(
  name='EncryptedPayload',
  full_name='EncryptedPayload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='EncryptedPayload.id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nonce', full_name='EncryptedPayload.nonce', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='EncryptedPayload.payload', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag', full_name='EncryptedPayload.tag', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=106,
)


_PLAINTEXTPAYLOAD = _descriptor.Descriptor(
  name='PlaintextPayload',
  full_name='PlaintextPayload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=108,
  serialized_end=126,
)


_SECBEACONREPORT = _descriptor.Descriptor(
  name='SecBeaconReport',
  full_name='SecBeaconReport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='clock', full_name='SecBeaconReport.clock', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='encrypted', full_name='SecBeaconReport.encrypted', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='plaintext', full_name='SecBeaconReport.plaintext', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='distance_cm', full_name='SecBeaconReport.distance_cm', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variance', full_name='SecBeaconReport.variance', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='SecBeaconReport.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=129,
  serialized_end=288,
)

_SECBEACONREPORT.fields_by_name['encrypted'].message_type = _ENCRYPTEDPAYLOAD
_SECBEACONREPORT.fields_by_name['plaintext'].message_type = _PLAINTEXTPAYLOAD
_SECBEACONREPORT.oneofs_by_name['data'].fields.append(
  _SECBEACONREPORT.fields_by_name['encrypted'])
_SECBEACONREPORT.fields_by_name['encrypted'].containing_oneof = _SECBEACONREPORT.oneofs_by_name['data']
_SECBEACONREPORT.oneofs_by_name['data'].fields.append(
  _SECBEACONREPORT.fields_by_name['plaintext'])
_SECBEACONREPORT.fields_by_name['plaintext'].containing_oneof = _SECBEACONREPORT.oneofs_by_name['data']
DESCRIPTOR.message_types_by_name['EncryptedPayload'] = _ENCRYPTEDPAYLOAD
DESCRIPTOR.message_types_by_name['PlaintextPayload'] = _PLAINTEXTPAYLOAD
DESCRIPTOR.message_types_by_name['SecBeaconReport'] = _SECBEACONREPORT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EncryptedPayload = _reflection.GeneratedProtocolMessageType('EncryptedPayload', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTEDPAYLOAD,
  '__module__' : 'c3loc.proto.secbeacon_pb2'
  # @@protoc_insertion_point(class_scope:EncryptedPayload)
  })
_sym_db.RegisterMessage(EncryptedPayload)

PlaintextPayload = _reflection.GeneratedProtocolMessageType('PlaintextPayload', (_message.Message,), {
  'DESCRIPTOR' : _PLAINTEXTPAYLOAD,
  '__module__' : 'c3loc.proto.secbeacon_pb2'
  # @@protoc_insertion_point(class_scope:PlaintextPayload)
  })
_sym_db.RegisterMessage(PlaintextPayload)

SecBeaconReport = _reflection.GeneratedProtocolMessageType('SecBeaconReport', (_message.Message,), {
  'DESCRIPTOR' : _SECBEACONREPORT,
  '__module__' : 'c3loc.proto.secbeacon_pb2'
  # @@protoc_insertion_point(class_scope:SecBeaconReport)
  })
_sym_db.RegisterMessage(SecBeaconReport)


# @@protoc_insertion_point(module_scope)
