{% extends 'base.html' %}

{% block title %}Proximity Demo - Index{% endblock title %}

{% block content %}
<div class="container">
    <h1>Proximity Demo</h1>
    <div class="row">
        <div class="three columns">
            <label for="showIBeacons">Show iBeacons?</label>
            <input type="checkbox" id="showIBeacons" checked="true"/>
        </div>
        <div class="three columns">
            <label for="showSmartRelay">Show SmartRelay?</label>
            <input type="checkbox" id="showSmartRelay" checked="true"/>
        </div>
        <div class="three columns">
            <label for="showLocationAnchors">Show Location Anchors?</label>
            <input type="checkbox" id="showLocationAnchors" checked="true"/>
        </div>
        <div class="three columns">
            <a class="button button-primary" href="zones">Add Zone</a>
        </div>
    </div>
    <div class="row twelve columns">
    <table id="prox_table">
        <thead>
        <tr>
            <th>Tag</th><th>Location</th><th>Distance</th><th>Last Seen</th><th></th>
        </tr>
        </thead>
        <tbody id="prox_table_body">
        </tbody>
    </table>
    </div>
</div>
{% endblock content %}

{% block js %}<script src="static/index.js"></script>{% endblock js%}
